library ac_dart.ac_jwt.product_host_token.test;

import 'package:atlassian_connect_jwt/src/product_host_token.dart';
import 'package:dart_jwt/dart_jwt.dart';

import 'package:unittest/unittest.dart';

void main()  {
  final String sharedSecret = '3ab90b11-d7bd-4097-958f-01b7ac4e985f';
  final String issuer = 'jira:ae390d29-31b2-4c12-a719-9df64e3e92b7';
  final String subject = 'admin';
  final DateTime expiry = DateTime.parse('2014-03-07 15:26:07.000');
  final DateTime issuedAt = DateTime.parse('2014-03-07 15:23:07.000');

  final JwaSignatureContext signatureContext = new JwaSignatureContext(sharedSecret);

  final uriStr = "/hello-world?lic=none&tz=Australia%2FSydney&cp=%2Fjira&user_key=&loc=en-US&user_id=&xdm_e=http%3A%2F%2Fstorm%3A2990&xdm_c=channel-servlet-hello-world&xdm_p=1";
  final uri = Uri.parse(uriStr);
  final lenientClaimSetValidationContext = new ProductHostClaimSetValidationContext.withFactory(
      defaultQshFactory(uri, 'GET'), expiryTolerance: const Duration(days: 365*1000));
  
  final JwtValidationContext validationContext = 
      new JwtValidationContext(signatureContext, lenientClaimSetValidationContext);
  
  group('[decode]', () {
    final String qsh = "eb4ec5f0980d01da306db89fa57dd163574667436dd720d5d9c5279c5b45f7a8";
    final String jwtStr = r'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjEzOTQxNjYzNjcsInN1YiI6ImFkbWluIiwiaXNzIjoiamlyYTphZTM5MGQyOS0zMWIyLTRjMTItYTcxOS05ZGY2NGUzZTkyYjciLCJxc2giOiJlYjRlYzVmMDk4MGQwMWRhMzA2ZGI4OWZhNTdkZDE2MzU3NDY2NzQzNmRkNzIwZDVkOWM1Mjc5YzViNDVmN2E4IiwiaWF0IjoxMzk0MTY2MTg3fQ.bR8Z0MIguOg6xgHiav0quun8kTqXzNUKMKym-PFjZvc';

    JsonWebToken jwt() => decodeProductHostToken(jwtStr);
    ProductHostClaimSet claimSet() => jwt().claimSet;
    
    group('[claimset]', () {
      test('issuer parses', () {
        expect(claimSet().issuer, equals(issuer));
      });
      test('subject parses', () {
        expect(claimSet().subject, equals(subject));
      });
      test('expiry parses', () {
        expect(claimSet().expiry, equals(expiry));
      });
      test('issuedAt parses', () {
        expect(claimSet().issuedAt, equals(issuedAt));
      });
      test('qsh parses', () {
        expect(claimSet().queryStringHash, equals(qsh));
      });
    });
    skip_group('[signature]', () {
      test('validates successfully with correct shared secret', () {
        Set<ConstraintViolation> violations = jwt().validate(validationContext);
        expect(violations, isEmpty);        
      });
    });
  });
  
  group('[encode]', () {
    final String qsh = "8063ff4ca1e41df7bc90c8ab6d0f6207d491cf6dad7c66ea797b4614b71922e9";
    final String jwtStr = r'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjEzOTQxNjYzNjcsInN1YiI6ImFkbWluIiwiaXNzIjoiamlyYTphZTM5MGQyOS0zMWIyLTRjMTItYTcxOS05ZGY2NGUzZTkyYjciLCJxc2giOiJlYjRlYzVmMDk4MGQwMWRhMzA2ZGI4OWZhNTdkZDE2MzU3NDY2NzQzNmRkNzIwZDVkOWM1Mjc5YzViNDVmN2E4IiwiaWF0IjoxMzk0MTY2MTg3fQ.bR8Z0MIguOg6xgHiav0quun8kTqXzNUKMKym-PFjZvc';

    final claimSet = new ProductHostClaimSet.mut(
        issuer: issuer,
        subject: subject,
        expiry: expiry,
        issuedAt: issuedAt,
        queryStringHash: qsh);
       
    JsonWebToken jwt() => new JsonWebToken.jws(claimSet, signatureContext);
    String encode() => jwt().encode();
    JsonWebToken parseEncoded() => decodeProductHostToken(encode(), validationContext: validationContext);
    ProductHostClaimSet roundtripClaimSet() => parseEncoded().claimSet;
    
    
    // need qsh etc for this to parse. Also order dependent in json so not useful
    skip_test('encodes correctly', () {
      expect(jwt().encode(), equals(jwtStr));
    });
    
    group('[roundtrip]', () {
      test('issuer matches', () {
        expect(roundtripClaimSet().issuer, equals(issuer));
      });
      test('subject matches', () {
        expect(roundtripClaimSet().subject, equals(subject));
      });
      test('expiry matches', () {
        expect(roundtripClaimSet().expiry, equals(expiry));
      });
      test('issuedAt matches', () {
        expect(roundtripClaimSet().issuedAt, equals(issuedAt));
      });
      test('queryStringHash matches', () {
        expect(roundtripClaimSet().queryStringHash, equals(qsh));
      });
    });
  });
  
  group('[validation]', () {
    const String correctQsh   = "8063ff4ca1e41df7bc90c8ab6d0f6207d491cf6dad7c66ea797b4614b71922e9";
    const String incorrectQsh = "8063ff4ca1e41df7bc90c8ab6d0f6207d491cf6dad7c66ea797b4614b71922e8";

    final claimSetValidationContext = new ProductHostClaimSetValidationContext.withFactory(
        defaultQshFactory(uri, 'GET'), expiryTolerance: const Duration(seconds: 30));

    ProductHostClaimSet claimSet(int secondsBeforeNow, String daQsh) => new ProductHostClaimSet.mut(
        issuer: issuer,
        subject: subject,
        expiry: new DateTime.now().subtract(new Duration(seconds: secondsBeforeNow)),
        issuedAt: issuedAt,
        queryStringHash: daQsh);
    
    Set<ConstraintViolation> violations(int secondsBeforeNow, String daQsh) => 
        claimSet(secondsBeforeNow, daQsh)
          .validate(claimSetValidationContext);
    
    test('fails validation if more than tolerance past expiry', () {
      expect(violations(31, correctQsh), isNot(isEmpty));
    });
  
    test('passes validation if no more than tolerance past expiry', () {
      expect(violations(30, correctQsh), isEmpty);
    });
    
    test("fails validation if query string hashes don't match", () {
      expect(violations(0, incorrectQsh), isNot(isEmpty));      
    });
    
    test("fails validation if query string hashes don't match - validate whole jwt", () {
      JsonWebToken jwt = new JsonWebToken.jws(claimSet(0, incorrectQsh), signatureContext);
      final validationContext = new JwtValidationContext(signatureContext, 
          new ProductHostClaimSetValidationContext.withFactory(defaultQshFactory(uri, 'GET')));
      expect(jwt.validate(validationContext), isNot(isEmpty));      
    });
  });
  
}

